@@ -5,25 +5,23 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002-2004 James Su <suzhe@tsinghua.org.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: scim_generic_table.cpp,v 1.4 2005/08/11 15:30:36 suzhe Exp $
+ * $Id: scim_generic_table.cpp,v 1.10 2006/01/12 08:43:29 suzhe Exp $
  *
  */
 
@@ -71,12 +69,12 @@ static inline String
 _get_param_portion (const String &str, const String &delim = "=")
 {
     String ret = str;
-    String::size_type pos = ret.find_first_of (String (" \t\v") + delim);
+    String::size_type pos = ret.find_first_of (delim);
 
     if (pos != String::npos)
         ret.erase (pos, String::npos);
 
-    return ret;
+    return _trim_blank (ret);
 }
 
 static inline String
@@ -89,18 +87,10 @@ _get_value_portion (const String &str, const String &delim = "=")
 
     if (pos != String::npos)
         ret.erase (0, pos + 1);
+    else
+        return String();
 
-    pos = ret.find_first_not_of(" \t\v");
-
-    if (pos != String::npos)
-        ret.erase (0, pos);
-
-    pos = ret.find_last_not_of(" \t\v");
-
-    if (pos != String::npos)
-        ret.erase (pos + 1, String::npos);
-
-    return ret;
+    return _trim_blank (ret);
 }
 
 static inline String  
@@ -174,6 +164,7 @@ GenericTableHeader::clear ()
     m_languages = String ();
     m_status_prompt = String ();
     m_valid_input_chars = String ();
+    m_key_end_chars = String ();
     m_single_wildcard_chars = String ();
     m_multi_wildcard_chars = String ();
     m_default_name = String ();
@@ -209,6 +200,12 @@ GenericTableHeader::is_valid_input_char (char input) const
     return std::binary_search (m_valid_input_chars.begin (), m_valid_input_chars.end (), input);
 }
 
+bool
+GenericTableHeader::is_key_end_char (char input) const
+{
+    return std::binary_search (m_key_end_chars.begin (), m_key_end_chars.end (), input);
+}
+
 bool
 GenericTableHeader::is_single_wildcard_char (char single) const
 {
@@ -245,6 +242,7 @@ GenericTableHeader::load (FILE *fp)
     String valuestr;
     String single_wildcard;
     String multi_wildcard;
+    String key_end_chars;
 
     std::vector <KeyEvent> split_keys;
 
@@ -261,8 +259,10 @@ GenericTableHeader::load (FILE *fp)
         paramstr = _get_param_portion (temp);
         valuestr = _get_value_portion (temp);
 
-        if (paramstr.length () == 0 || valuestr.length () == 0)
-            return false;
+        if (paramstr.length () == 0 && valuestr.length () == 0) {
+            std::cerr << "Invalid line in header: " << temp << "\n";
+            continue;
+        }
 
         if (paramstr == "NAME") { // Get table default name.
             m_default_name = valuestr;
@@ -350,6 +350,8 @@ GenericTableHeader::load (FILE *fp)
                 m_def_full_width_letter = false;
         } else if (paramstr == "VALID_INPUT_CHARS") { //Get valid input chars.
             m_valid_input_chars = valuestr;
+        } else if (paramstr == "KEY_END_CHARS") { //Get valid input chars.
+            key_end_chars = valuestr;
         } else if (paramstr == "SINGLE_WILDCARD_CHAR") { //Get single wildcard char.
             single_wildcard = valuestr;
         } else if (paramstr == "MULTI_WILDCARD_CHAR") { //Get multi wildcard char.
@@ -366,6 +368,12 @@ GenericTableHeader::load (FILE *fp)
             scim_string_to_key_list (m_page_up_keys, valuestr);
         } else if (paramstr == "PAGE_DOWN_KEYS") {
             scim_string_to_key_list (m_page_down_keys, valuestr);
+        } else if (paramstr == "MODE_SWITCH_KEYS") {
+            scim_string_to_key_list (m_mode_switch_keys, valuestr);
+        } else if (paramstr == "FULL_WIDTH_PUNCT_KEYS") {
+            scim_string_to_key_list (m_full_width_punct_keys, valuestr);
+        } else if (paramstr == "FULL_WIDTH_LETTER_KEYS") {
+            scim_string_to_key_list (m_full_width_letter_keys, valuestr);
         } else if (paramstr == "MAX_KEY_LENGTH") {
             m_max_key_length = atoi (valuestr.c_str ());
         } else if (paramstr == "BEGIN_CHAR_PROMPTS_DEFINITION") { //Read char names.
@@ -381,11 +389,11 @@ GenericTableHeader::load (FILE *fp)
                 m_char_prompts.push_back (String (temp));
             }
         } else {
-            return false;
+            std::cerr << "Invalid line in header: " << temp << "\n";
         }
     }
 
-    //Post process inputed information.
+    //Post process inputted information.
 
     if (!m_uuid.length () || !m_serial_number.length ())
         return false;
@@ -404,21 +412,28 @@ GenericTableHeader::load (FILE *fp)
     }
 
     size_t i;
-    for (i=0; i<split_keys.size (); ++i)
-        if (m_valid_input_chars.find (split_keys [i].get_ascii_code ()) == String::npos)
-            m_split_keys.push_back (split_keys [i]);
-
-    for (i=0; i<single_wildcard.length (); ++i)
-        if (m_valid_input_chars.find (single_wildcard [i]) == String::npos)
-            m_single_wildcard_chars.push_back (single_wildcard [i]);
-
-    for (i=0; i<multi_wildcard.length (); ++i)
-        if (m_valid_input_chars.find (multi_wildcard [i]) == String::npos &&
-            m_single_wildcard_chars.find (multi_wildcard [i]) == String::npos)
-            m_multi_wildcard_chars.push_back (multi_wildcard [i]);
 
     std::sort (m_valid_input_chars.begin (), m_valid_input_chars.end ());
+
+    for (i=0; i<single_wildcard.length (); ++i)
+        if (!is_valid_input_char (single_wildcard [i]))
+            m_single_wildcard_chars.push_back (single_wildcard [i]);
+
     std::sort (m_single_wildcard_chars.begin (), m_single_wildcard_chars.end ());
+
+    for (i=0; i<multi_wildcard.length (); ++i)
+        if (!is_valid_input_char (multi_wildcard [i]) && !is_single_wildcard_char (multi_wildcard [i]))
+            m_multi_wildcard_chars.push_back (multi_wildcard [i]);
+
+    for (i=0; i<key_end_chars.length (); ++i)
+        if (is_valid_input_char (key_end_chars [i]))
+            m_key_end_chars.push_back (key_end_chars [i]);
+
+    for (i=0; i<split_keys.size (); ++i)
+        if (!is_valid_input_char (split_keys [i].get_ascii_code ()))
+            m_split_keys.push_back (split_keys [i]);
+
+    std::sort (m_key_end_chars.begin (), m_key_end_chars.end ());
     std::sort (m_multi_wildcard_chars.begin (), m_multi_wildcard_chars.end ());
     std::sort (m_char_prompts.begin (), m_char_prompts.end ());
     std::sort (m_local_names.begin (), m_local_names.end ());
@@ -476,6 +491,11 @@ GenericTableHeader::save (FILE *fp)
     fprintf (fp, "KEYBOARD_LAYOUT = %s\n", scim_keyboard_layout_to_string (m_keyboard_layout).c_str ());
     fprintf (fp, "VALID_INPUT_CHARS = %s\n", m_valid_input_chars.c_str ());
 
+    if (m_key_end_chars.length ())
+        fprintf (fp, "KEY_END_CHARS = %s\n", m_key_end_chars.c_str ());
+    else
+        fprintf (fp, "### KEY_END_CHARS =\n");
+
     if (m_single_wildcard_chars.length ())
         fprintf (fp, "SINGLE_WILDCARD_CHAR = %s\n", m_single_wildcard_chars.c_str ());
     else
@@ -522,6 +542,24 @@ GenericTableHeader::save (FILE *fp)
     else
         fprintf (fp, "### PAGE_DOWN_KEYS =\n");
 
+    scim_key_list_to_string (temp, m_mode_switch_keys);
+    if (temp.length ())
+        fprintf (fp, "MODE_SWITCH_KEYS = %s\n", temp.c_str ());
+    else
+        fprintf (fp, "### MODE_SWITCH_KEYS =\n");
+
+    scim_key_list_to_string (temp, m_full_width_punct_keys);
+    if (temp.length ())
+        fprintf (fp, "FULL_WIDTH_PUNCT_KEYS = %s\n", temp.c_str ());
+    else
+        fprintf (fp, "### FULL_WIDTH_PUNCT_KEYS =\n");
+
+    scim_key_list_to_string (temp, m_full_width_letter_keys);
+    if (temp.length ())
+        fprintf (fp, "FULL_WIDTH_LETTER_KEYS = %s\n", temp.c_str ());
+    else
+        fprintf (fp, "### FULL_WIDTH_LETTER_KEYS =\n");
+
     fprintf (fp, "MAX_KEY_LENGTH = %u\n", m_max_key_length);
 
     fprintf (fp, "SHOW_KEY_PROMPT = %s\n", (m_show_key_prompt?"TRUE":"FALSE"));
@@ -817,10 +855,7 @@ GenericTableContent::GenericTableContent ()
 }
 
 bool
-GenericTableContent::init (const String & input_chars,
-                           const String & single_wildcard,
-                           const String & multi_wildcard,
-                           size_t         max_key_length)
+GenericTableContent::init (const GenericTableHeader &header)
 {
     size_t i;
 
@@ -832,7 +867,7 @@ GenericTableContent::init (const String & input_chars,
     m_single_wildcard_char = 0;
     m_multi_wildcard_char = 0;
 
-    m_max_key_length = std::min (max_key_length, (size_t) SCIM_GT_MAX_KEY_LENGTH);
+    m_max_key_length = std::min (header.get_max_key_length (), (size_t) SCIM_GT_MAX_KEY_LENGTH);
 
     if (m_max_key_length) {
         if (m_offsets) delete [] m_offsets;
@@ -847,11 +882,17 @@ GenericTableContent::init (const String & input_chars,
             return false;
         }
 
-        for (i = 0; i < input_chars.length (); ++ i)
-            m_char_attrs [(size_t) ((unsigned char) input_chars [i])] = 1;
+        String chars = header.get_valid_input_chars ();
 
-        set_single_wildcard_chars (single_wildcard);
-        set_multi_wildcard_chars (multi_wildcard);
+        for (i = 0; i < chars.length (); ++ i)
+            m_char_attrs [(size_t) ((unsigned char) chars [i])] = GT_CHAR_ATTR_VALID_CHAR;
+
+        chars = header.get_key_end_chars ();
+        for (i = 0; i < chars.length (); ++ i)
+            m_char_attrs [(size_t) ((unsigned char) chars [i])] |= GT_CHAR_ATTR_KEY_END_CHAR;
+
+        set_single_wildcard_chars (header.get_single_wildcard_chars ());
+        set_multi_wildcard_chars (header.get_multi_wildcard_chars ());
 
         return true;
     }
@@ -876,7 +917,7 @@ GenericTableContent::set_single_wildcard_chars (const String &single)
     if (m_max_key_length) {
         size_t i;
         for (i = 0; i < 256; ++ i) {
-            if (m_char_attrs [i] == 2)
+            if (is_single_wildcard_char (m_char_attrs [i]))
                 m_char_attrs [i] = 0;
         }
 
@@ -884,10 +925,10 @@ GenericTableContent::set_single_wildcard_chars (const String &single)
 
         for (i = 0; i < single.length (); ++ i)
             if (!m_char_attrs [(size_t) ((unsigned char) single[i])])
-                m_char_attrs [(size_t) ((unsigned char) single[i])] = 2;
+                m_char_attrs [(size_t) ((unsigned char) single[i])] = GT_CHAR_ATTR_SINGLE_WILDCARD;
 
         for (i = 0; i < 256; ++ i) {
-            if (m_char_attrs [i] == 2) {
+            if (m_char_attrs [i] == GT_CHAR_ATTR_SINGLE_WILDCARD) {
                 m_single_wildcard_char = (char) i;
                 break;
             }
@@ -895,10 +936,10 @@ GenericTableContent::set_single_wildcard_chars (const String &single)
 
         //No defined single wildcard char, choose one
         if (!m_single_wildcard_char) {
-            for (i = 2; i < 256; ++ i) {
+            for (i = 1; i < 256; ++ i) {
                 if (!m_char_attrs [i]) {
                     m_single_wildcard_char = (char) i;
-                    m_char_attrs [i] = 2;
+                    m_char_attrs [i] = GT_CHAR_ATTR_SINGLE_WILDCARD;
                     break;
                 }
             }
@@ -912,7 +953,7 @@ GenericTableContent::set_multi_wildcard_chars (const String &multi)
     if (m_max_key_length) {
         size_t i;
         for (i = 0; i < 256; ++ i) {
-            if (m_char_attrs [i] == 3)
+            if (is_multi_wildcard_char (m_char_attrs [i]))
                 m_char_attrs [i] = 0;
         }
 
@@ -920,10 +961,10 @@ GenericTableContent::set_multi_wildcard_chars (const String &multi)
 
         for (i = 0; i < multi.length (); ++ i)
             if (!m_char_attrs [(uint32) multi[i]])
-                m_char_attrs [(uint32) multi[i]] = 3;
+                m_char_attrs [(uint32) multi[i]] = GT_CHAR_ATTR_MULTI_WILDCARD;
 
         for (i = 0; i < 256; ++ i) {
-            if (m_char_attrs [i] == 3) {
+            if (m_char_attrs [i] == GT_CHAR_ATTR_MULTI_WILDCARD) {
                 m_multi_wildcard_char = (char) i;
                 break;
             }
@@ -931,10 +972,10 @@ GenericTableContent::set_multi_wildcard_chars (const String &multi)
 
         //No defined multi wildcard char, choose one
         if (!m_multi_wildcard_char) {
-            for (i = 3; i < 256; ++ i) {
+            for (i = 1; i < 256; ++ i) {
                 if (!m_char_attrs [i]) {
                     m_multi_wildcard_char = (char) i;
-                    m_char_attrs [i] = 3;
+                    m_char_attrs [i] = GT_CHAR_ATTR_MULTI_WILDCARD;
                     break;
                 }
             }
@@ -988,12 +1029,18 @@ GenericTableContent::load_text (FILE *fp)
 
     std::vector <String> keys;
     std::vector <String> phrases;
-    std::vector <uint16> frequencies;
+    std::vector <uint32> frequencies;
 
     uint32 freq;
+    uint32 lengths_count [SCIM_GT_MAX_KEY_LENGTH];
+
+    size_t i;
 
     clear ();
 
+    for (i = 0; i < SCIM_GT_MAX_KEY_LENGTH; ++i)
+        lengths_count [i] = 0;
+
     if (_get_line (fp) != String ("BEGIN_TABLE"))
         return false;
 
@@ -1005,9 +1052,11 @@ GenericTableContent::load_text (FILE *fp)
  
         paramstr = _get_param_portion (temp, " \t");
         valuestr = _get_value_portion (temp, " \t");
- 
-        if (paramstr.length () == 0 || valuestr.length () == 0)
-            return false;
+
+        if (paramstr.length () == 0 || valuestr.length () == 0) {
+            std::cerr << "Invalid line in content: " << temp << "\n";
+            continue;
+        }
 
         phrasestr = _get_param_portion (valuestr, " \t");
         freqstr   = _get_value_portion (valuestr, " \t");
@@ -1022,9 +1071,7 @@ GenericTableContent::load_text (FILE *fp)
         if (freqstr.length ())
             freq = atoi (freqstr.c_str ());
         else
-            freq = 0;
-
-        if (freq > 65535) freq = 65535;
+            freq = ~0;
 
         phrasestr = utf8_wcstombs (wide_phrase);
         while (phrasestr.length () > 255) {
@@ -1039,12 +1086,21 @@ GenericTableContent::load_text (FILE *fp)
             keys.push_back (paramstr);
             phrases.push_back (phrasestr);
             frequencies.push_back (freq);
+            ++lengths_count [paramstr.length ()];
         }
     }
 
+    // Use phrase sequence as frequency to retain the correct order, if the frequency is missing.
+    for (i = 0; i < frequencies.size (); ++i) {
+        if (frequencies [i] == (uint32) ~0)
+            frequencies [i] = lengths_count [keys [i].length ()] --;
+
+        if (frequencies [i] > 65535)
+            frequencies [i] = 65535;
+    }
+
     // Calculate the content size.
     uint32 content_size = 0;
-    size_t i;
     for (i = 0; i < keys.size (); ++i) {
         content_size += keys [i].length ();
         content_size += phrases [i].length ();
@@ -1444,9 +1500,9 @@ GenericTableContent::is_valid_key (const String & key) const
         return false;
 
     for (String::const_iterator i = key.begin (); i != key.end (); ++ i) {
-        if (!m_char_attrs [(size_t) ((unsigned char) *i)])
+        if (!is_defined_char (*i))
             return false;
-        else if (m_char_attrs [(size_t) ((unsigned char) *i)] == 3)
+        else if (is_multi_wildcard_char (*i))
             multi_wildcard_count ++;
     }
 
@@ -1457,7 +1513,7 @@ bool
 GenericTableContent::is_wildcard_key (const String & key) const
 {
     for (String::const_iterator i = key.begin (); i != key.end (); ++ i) {
-        if (m_char_attrs [(size_t) ((unsigned char) *i)] > 1)
+        if (is_wildcard_char (*i))
             return true;
     }
 
@@ -1468,7 +1524,7 @@ bool
 GenericTableContent::is_pure_wildcard_key (const String & key) const
 {
     for (String::const_iterator i = key.begin (); i != key.end (); ++ i) {
-        if (m_char_attrs [(size_t) ((unsigned char) *i)] == 1)
+        if (!is_wildcard_char (*i))
             return false;
     }
 
@@ -1482,7 +1538,7 @@ GenericTableContent::is_valid_no_wildcard_key (const String & key) const
         return false;
 
     for (String::const_iterator i = key.begin (); i != key.end (); ++ i)
-        if (m_char_attrs [(size_t) ((unsigned char) *i)] != 1)
+        if (is_wildcard_char (*i) || !is_valid_char (*i))
             return false;
 
     return true;
@@ -1551,9 +1607,9 @@ GenericTableContent::sort_all_offsets ()
 {
     if (valid ()) {
         for (size_t i = 0; i < m_max_key_length; ++i)
-            std::sort (m_offsets [i].begin (),
-                       m_offsets [i].end (),
-                       OffsetLessByKeyFixedLen (m_content, i + 1));
+            std::stable_sort (m_offsets [i].begin (),
+                              m_offsets [i].end (),
+                              OffsetLessByKeyFixedLen (m_content, i + 1));
         init_all_offsets_attrs ();
     }
 }
@@ -1672,7 +1728,7 @@ GenericTableContent::find_no_wildcard_key (std::vector <uint32> &offsets, const
         for (; i != m_offsets_attrs [len].end (); ++ i) {
             if (i->mask.check (key)) {
                 if (i->dirty) {
-                    std::sort (m_offsets [len].begin () + i->begin,
+                    std::stable_sort (m_offsets [len].begin () + i->begin,
                                m_offsets [len].begin () + i->end,
                                sort_op);
                     i->dirty = false;
@@ -1859,9 +1915,9 @@ GenericTableContent::search_no_wildcard_key (const String &key, size_t len) cons
         for (; i != m_offsets_attrs [len].end (); ++ i) {
             if (i->mask.check (key)) {
                 if (i->dirty) {
-                    std::sort (m_offsets [len].begin () + i->begin,
-                               m_offsets [len].begin () + i->end,
-                               sort_op);
+                    std::stable_sort (m_offsets [len].begin () + i->begin,
+                                      m_offsets [len].begin () + i->end,
+                                      sort_op);
                     i->dirty = false;
                 }
 
@@ -1942,12 +1998,21 @@ GenericTableContent::delete_phrase (uint32 offset)
                                offset);
         if (lb < ub) {
             m_offsets [len - 1].erase (lb);
+
+            std::stable_sort (m_offsets [len - 1].begin (),
+                              m_offsets [len - 1].end (),
+                              OffsetLessByKeyFixedLen (m_content, len));
+
             init_offsets_attrs (len);
 
             m_updated = true;
 
             return true;
         }
+
+        std::stable_sort (m_offsets [len - 1].begin (),
+                          m_offsets [len - 1].end (),
+                          OffsetLessByKeyFixedLen (m_content, len));
     }
     return false;
 }
@@ -1981,9 +2046,9 @@ GenericTableContent::add_phrase (const String &key, const WideString &phrase, in
             // Added the offset.
             m_offsets [key_len - 1].push_back (m_content_size);
 
-            std::sort (m_offsets [key_len - 1].begin (),
-                       m_offsets [key_len - 1].end (),
-                       OffsetLessByKeyFixedLen (m_content, key_len));
+            std::stable_sort (m_offsets [key_len - 1].begin (),
+                              m_offsets [key_len - 1].end (),
+                              OffsetLessByKeyFixedLen (m_content, key_len));
 
             m_content_size += add_size;
 
@@ -2013,7 +2078,7 @@ GenericTableContent::init_offsets_by_phrases () const
                                      m_offsets [i].end ());
     }
 
-    std::sort (m_offsets_by_phrases.begin (), m_offsets_by_phrases.end (), OffsetLessByPhrase (m_content));
+    std::stable_sort (m_offsets_by_phrases.begin (), m_offsets_by_phrases.end (), OffsetLessByPhrase (m_content));
 
     m_offsets_by_phrases_inited = true;
 }
@@ -2130,16 +2195,10 @@ GenericTableLibrary::load_header ()
         ok = header.load (fp);
 
     if (ok)
-        ok = m_sys_content.init (header.get_valid_input_chars (),
-                                 header.get_single_wildcard_chars (),
-                                 header.get_multi_wildcard_chars (),
-                                 header.get_max_key_length ());
+        ok = m_sys_content.init (header);
 
     if (ok)
-        ok = m_usr_content.init (header.get_valid_input_chars (),
-                                 header.get_single_wildcard_chars (),
-                                 header.get_multi_wildcard_chars (),
-                                 header.get_max_key_length ());
+        ok = m_usr_content.init (header);
 
     if (ok) {
         m_header = header;
