@@ -5,25 +5,23 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002-2004 James Su <suzhe@tsinghua.org.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: scim_table_imengine_setup.cpp,v 1.2 2005/01/06 02:13:37 suzhe Exp $
+ * $Id: scim_table_imengine_setup.cpp,v 1.4 2005/10/26 07:53:53 suzhe Exp $
  *
  */
 
@@ -1508,7 +1506,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
     KeyboardConfigData split_keys  = {NULL,
                                       _("Split Keys:"),
                                       _("Split Keys:"),
-                                      _("The key strokes to split inputed string."),
+                                      _("The key strokes to split inputted string."),
                                       NULL,
                                       NULL,
                                       ""};
@@ -1524,7 +1522,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
     KeyboardConfigData forward_keys= {NULL,
                                       _("Forward Keys:"),
                                       _("Forward Keys:"),
-                                      _("The key strokes to forward inputed string to client."),
+                                      _("The key strokes to forward inputted string to client."),
                                       NULL,
                                       NULL,
                                       ""};
@@ -1917,7 +1915,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
  
         gtk_tooltips_set_tip (tooltips, toggle_auto_wildcard,
                               _("If true then a multi wildcard char will be append to "
-                                "the end of inputed key string when searching phrases."), NULL);
+                                "the end of inputted key string when searching phrases."), NULL);
 
         ++ row;
 
@@ -1964,7 +1962,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
                           0);
  
         gtk_tooltips_set_tip (tooltips, toggle_auto_split,
-                              _("If true then the inputed key string will be "
+                              _("If true then the inputted key string will be "
                                 "splitted automatically when necessary."), NULL);
 
         ++ row;
@@ -2088,7 +2086,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
                           0);
   
         gtk_tooltips_set_tip (tooltips, toggle_def_full_width_punct,
-                              _("If true then full width punctuations will be inputed by default."), NULL);
+                              _("If true then full width punctuations will be inputted by default."), NULL);
 
         ++ row;
 
@@ -2111,7 +2109,7 @@ run_table_properties_dialog (GenericTableLibrary *lib, TablePropertiesData &data
                           0);
   
         gtk_tooltips_set_tip (tooltips, toggle_def_full_width_letter,
-                              _("If true then full width letters will be inputed by default."), NULL);
+                              _("If true then full width letters will be inputted by default."), NULL);
 
         // action buttons
         dialog_action_area = GTK_DIALOG (dialog)->action_area;
